{
Beat sequencer by Owen Trueblood
Controls-
  -Arrow keys to move cursor
  -Enter to place note
  -Space to listen to song
  -Escape key to go back to editing from listening

  This is a template for a beat sequencer. As long as I am
  given credit for making the original template, this code
  is free to use in any way you like.

  Try and make modifications! Cooler sounds or maybe even
  make use of the VocalTract object or the HSS
}

CON

_clkmode = xtal1 + pll16x 
_xinfreq = 5_000_000

'For synth
Audio_Pins = %00000000_00000000_00001100_00000000
Audio_Ctra = %0_00110_000_00000000_000000_000_001010
Audio_Pin  = 10 ''for freq gen
A_Counter  = "B"

keyboard   = 26 '' I/O for keyboard connection
video      = 12 '' I/O for composite video connection

'Colors stolen from PropDOS
BLACK             = $02 
GREY              = $03
LWHITE            = $06
BWHITE            = $07
LMAGENTA          = $A3 
WHITE             = $AF
RED               = $A2
LRED              = $1A
LPURPLE           = $1B
PURPLE            = $03 
LGREEN            = $1C
SKYBLUE           = $1D
YELLOW            = $1E
BYELLOW           = $A6
LGREY             = $1F
DGREY             = $08
GREEN             = $A4
ORANGE            = $06
BLUE              = $A1 
CYAN              = $1C
MAGENTA           = $FC
LYELLOW           = $1E

'Carriage return value
CR    = $0D      

'Keycodes
UP    = 194
DOWN  = 195
LEFT  = 192
RIGHT = 193
ENTER = 13
SPACE = 32

'Amount of time between notes
DELAY = 500   

OBJ

Freq  : "Synth"
pause : "pause"
text  : "aigeneric_driver_mod"  
key   : "comboKeyboard"

VAR

  long grid[12]

  long pos_x
  long pos_y
 
PUB start | i, k, keyval
  'Start pause, the keyboard, and video
  pause.start
  key.start(keyboard)
  text.start(video)

  'Splash screen
  Splash
  
  'Initialize screen and some vars
  Init
  
  'Enter edit mode
  Edit  

PRI Splash | keyval

  text.color(LWHITE)
  text.str(string("Propeller Beat Sequencer",CR))
  text.str(string("Created by Owen Trueblood"))

  'Pause until a key is pressed and then clear the screen
  keyval := 0
  repeat while keyval == 0
    keyval := key.key

  'Clear the screen
  text.cls

PRI Init | i, k

  'Clear the beat list
  repeat i from 0 to 11
    grid[i] := 0
  
  'Change the color
  text.color(WHITE)

  'Draw the notes on the side of the screen
  repeat i from 0 to 24
    text.dec(i*25)
    text.str(string(CR))

  'Draw the beat table
  repeat i from 0 to 24
    repeat k from 3 to 36 step 3
      text.poke(k, i, WHITE, 91)
      text.poke(k+1, i, WHITE, 32)
      text.poke(k+2, i, WHITE, 93)

  'Draw the beats
  repeat i from 0 to 11
    text.poke((i*3)+4, grid[i]/25, LWHITE, 42)

  'Initialize the cursor position
  pos_x := 0
  pos_y := 0

PRI Edit | i, keyval, check
  'Editing Mode
  text.poke((pos_x*3)+4, pos_y, LWHITE, 88)
  repeat
    keyval := key.key

    'Thank goodness TI BASIC is so slow, never would have found this
    'more efficient method of control (although has much more code)

    'Handle keypresses
    if keyval == UP
      if pos_y>0
        if grid[pos_x] == pos_y*25
          text.poke((pos_x*3)+4, pos_y, LWHITE, 42)
        else
          text.poke((pos_x*3)+4, pos_y, LWHITE, 32)
          
        pos_y--
        text.poke((pos_x*3)+4, pos_y, LWHITE, 88)
        
    if keyval == DOWN
      if pos_y<24
        if grid[pos_x] == pos_y*25  
          text.poke((pos_x*3)+4, pos_y, LWHITE, 42)
        else
          text.poke((pos_x*3)+4, pos_y, LWHITE, 32)
          
        pos_y++
        text.poke((pos_x*3)+4, pos_y, LWHITE, 88)
        
    if keyval == LEFT
      if pos_x>0
        if grid[pos_x] == pos_y*25  
          text.poke((pos_x*3)+4, pos_y, LWHITE, 42)
        else
          text.poke((pos_x*3)+4, pos_y, LWHITE, 32)
          
        pos_x--
        text.poke((pos_x*3)+4, pos_y, LWHITE, 88)
        
    if keyval == RIGHT
      if pos_x<11
        if grid[pos_x] == pos_y*25  
          text.poke((pos_x*3)+4, pos_y, LWHITE, 42)
        else
          text.poke((pos_x*3)+4, pos_y, LWHITE, 32)
            
        pos_x++
        text.poke((pos_x*3)+4, pos_y, LWHITE, 88)

    if keyval == ENTER
      text.poke((pos_x*3)+4, grid[pos_x]/25, LWHITE, 32)
      text.poke((pos_x*3)+4, pos_y, LWHITE, 42)
        
      grid[pos_x] := pos_y*25

    if keyval == SPACE
      Listen    
    
PRI Listen | pos

  'Loop and change the color of the note so the user can track the progress
  'of the song
  repeat
    repeat pos from 0 to 11
      'So the last grid position gets erased      
      if pos == 0
        text.poke(37, grid[11]/25, LWHITE, 42)
      else
        text.poke(((pos-1)*3)+4, grid[pos-1]/25, LWHITE, 42)

      'Exit back to editing  
      if key.key == 203
        'Don't forget to turn off the sound!
        Freq.Synth(A_Counter,  Audio_Pin, 0)

        text.poke((pos*3)+4, grid[pos]/25, LWHITE, 42)
        
        if pos == 0
          text.poke(37, grid[11]/25, LWHITE, 42)
        else
          text.poke(((pos-1)*3)+4, grid[pos-1]/25, LWHITE, 42)
          
        Edit
        
      text.poke((pos*3)+4, grid[pos]/25, RED, 42)
      Freq.Synth(A_Counter,  Audio_Pin, grid[pos])
      pause.pause(DELAY)